import subprocess
import os
import stat
import glob
import re


def call(*args, env={}):
    e = os.environ.copy()
    for k, v in env.items():
        e[k] = v
    return subprocess.run(args, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL, env=e).returncode


def is_service(name):
    return call("systemctl", "is-enabled", name) == 0


def disable_service(name):
    return call("systemctl", "disable", name) == 0


def enable_service(name):
    return call("systemctl", "enable", name) == 0


def is_installed(name):
    return call("dpkg", "-s", name) == 0


def install(*names):
    return call("apt-get", "install", "-yq", *names, env={"DEBIAN_FRONTEND": "noninteractive"}) == 0


def uninstall(name):
    return call("apt-get", "purge", "-yq", name) == 0


def write_sysctl(name):
    call("sysctl", "-w", name)


def is_empty(file):
    return os.path.isfile(file) and os.stat(file).st_size == 0


def search_command_result(command, pattern):
    if type(pattern) == str:
        pattern = re.compile(pattern)
    return (pattern.search(subprocess.getoutput(command))) != None


def exact_permission(file, permission, owner_id, group_id=None):
    if not os.path.isfile(file):
        return True
    if group_id is None:
        group_id = owner_id

    s = os.stat(file)
    return s.st_mode & 0o777 == permission and s.st_uid == owner_id and s.st_gid == group_id


def at_least_permission(file, permission, owner_id, group_id=None):
    if not os.path.isfile(file):
        return True
    if group_id is None:
        group_id = owner_id

    s = os.stat(file)
    return s.st_mode & 0o777 <= permission and s.st_uid == owner_id and s.st_gid == group_id


def change_permissions(file, permission, user_id=-1, group_id=-1):
    os.chmod(file, permission)
    os.chown(file, user_id, group_id)
    return True


def grep(pattern, *files):
    if type(pattern) == str:
        pattern = re.compile(pattern, re.MULTILINE)

    for i in files:
        for j in glob.glob(i, recursive=True):
            if os.path.isfile(j):
                with open(j, "r") as f:
                    if grep_readable(pattern, f):
                        return True

    return False


def sed(pattern, replacement, *files, append=True):
    if type(pattern) == str:
        pattern = re.compile(pattern, re.MULTILINE)

    for i in files:
        for j in glob.glob(i):
            with open(j, "r+") as f:
                data = f.read()
                new_data = re.sub(pattern, replacement, data)
                f.seek(0)
                f.truncate()
                f.write(new_data)
                if append and new_data == data:
                    f.write("\n")
                    f.write(replacement)

    return True


def grep_readable(pattern, *readables):
    if type(pattern) == str:
        pattern = re.compile(pattern, re.MULTILINE)

    return any(re.search(pattern, i.read()) for i in readables)


def is_aide_checked():
    return call("bash", "-c", "crontab -u root -l | grep 'aide'") == 0


def fix_aide():
    return call("bash", "-c", '{ crontab -u root -l; echo "0 5 * * * /usr/bin/aide.wrapper --config /etc/aide/aide.conf --check"; } | crontab -u root - ') == 0
